Index: if_run.c
===================================================================
--- if_run.c	(revision 251736)
+++ if_run.c	(working copy)
@@ -52,6 +52,7 @@
 #include <net/if_dl.h>
 #include <net/if_media.h>
 #include <net/if_types.h>
+#include <net/vnet.h>
 
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -606,9 +607,12 @@
 
 	RUN_UNLOCK(sc);
 
+	CURVNET_SET(vnet0);
+
 	ifp = sc->sc_ifp = if_alloc(IFT_IEEE80211);
 	if (ifp == NULL) {
 		device_printf(sc->sc_dev, "can not if_alloc()\n");
+		CURVNET_RESTORE();
 		goto detach;
 	}
 	ic = ifp->if_l2com;
@@ -702,6 +706,8 @@
 	if (bootverbose)
 		ieee80211_announce(ic);
 
+	CURVNET_RESTORE();
+
 	return (0);
 
 detach:
@@ -739,8 +745,10 @@
 		usb_callout_drain(&sc->ratectl_ch);
 		ieee80211_draintask(ic, &sc->cmdq_task);
 		ieee80211_draintask(ic, &sc->ratectl_task);
+		CURVNET_SET(ifp->if_vnet);
 		ieee80211_ifdetach(ic);
 		if_free(ifp);
+		CURVNET_RESTORE();
 	}
 
 	mtx_destroy(&sc->sc_mtx);
@@ -2528,13 +2536,10 @@
 	m->m_pkthdr.rcvif = ifp;
 	m->m_pkthdr.len = m->m_len = len;
 
-	if (ni != NULL) {
-		(void)ieee80211_input(ni, m, rssi, nf);
-		ieee80211_free_node(ni);
-	} else {
-		(void)ieee80211_input_all(ic, m, rssi, nf);
-	}
-
+	/*
+	 * DAAN: fill-in tap header BEFORE calling ieee80211_input*() so the
+	 * user will see the actual data that belongs to THIS packet..
+	 */
 	if (__predict_false(ieee80211_radiotap_active(ic))) {
 		struct run_rx_radiotap_header *tap = &sc->sc_rxtap;
 
@@ -2571,6 +2576,13 @@
 			break;
 		}
 	}
+
+	if (ni != NULL) {
+		(void)ieee80211_input(ni, m, rssi, nf);
+		ieee80211_free_node(ni);
+	} else {
+		(void)ieee80211_input_all(ic, m, rssi, nf);
+	}
 }
 
 static void
